@@ -1,14 +1,12 @@
-
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <math.h>
 #include <gmp.h>
-
 #include "ptypes.h"
-#include "gmp_main.h"
+
+#include "bls75.h"
+#include "primality.h"
 #include "prime_iterator.h"
 #include "small_factor.h"
+#include "factor.h"
 #include "simpqs.h"
 #include "ecm.h"
 #define _GMP_ECM_FACTOR(n, f, b1, ncurves) \
@@ -529,6 +527,8 @@ int _GMP_primality_bls_nm1(mpz_t n, int effort, char** prooftextptr)
      * since we did not perform an exhaustive search.  It would be quite
      * unusual to find a prime that didn't have an 'a' in the first 10,000
      * primes, but it could happen.  It's a "dubiously prime" :) */
+    if (get_verbose_level() > 0)
+      printf("N-1 factored but failed to prove.  Perhaps composite.\n");
     mpz_clear(p);
     mpz_clear(ap);
   }
