@@ -5,7 +5,7 @@ use Carp qw/croak confess carp/;
 
 BEGIN {
   $Math::Prime::Util::GMP::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::GMP::VERSION = '0.35';
+  $Math::Prime::Util::GMP::VERSION = '0.36';
 }
 
 # parent is cleaner, and in the Perl 5.10.1 / 5.12.0 core, but not earlier.
@@ -40,6 +40,7 @@ our @EXPORT_OK = qw(
                      sieve_primes
                      sieve_twin_primes
                      sieve_prime_cluster
+                     sieve_range
                      next_prime
                      prev_prime
                      trial_factor
@@ -61,7 +62,8 @@ our @EXPORT_OK = qw(
                      factorial
                      consecutive_integer_lcm
                      partitions bernfrac harmfrac harmreal stirling
-                     gcd lcm kronecker valuation invmod binomial gcdext
+                     gcd lcm kronecker valuation binomial gcdext
+                     invmod sqrtmod addmod mulmod divmod powmod
                      vecsum vecprod
                      exp_mangoldt
                      liouville
@@ -69,6 +71,7 @@ our @EXPORT_OK = qw(
                      jordan_totient
                      carmichael_lambda
                      is_power
+                     is_primitive_root
                      znorder
                      znprimroot
                      ramanujan_tau
@@ -166,7 +169,7 @@ __END__
 
 =encoding utf8
 
-=for stopwords Möbius Deléglise Bézout gcdext vecsum vecprod moebius totient liouville znorder znprimroot bernfrac harmfrac harmreal stirling lucasu lucasv OpenPFGW gmpy2 nonresidue chinese tuplets
+=for stopwords Möbius Deléglise Bézout gcdext vecsum vecprod moebius totient liouville znorder znprimroot bernfrac harmfrac harmreal stirling lucasu lucasv OpenPFGW gmpy2 nonresidue chinese tuplets sqrtmod addmod mulmod powmod divmod
 
 =head1 NAME
 
@@ -175,7 +178,7 @@ Math::Prime::Util::GMP - Utilities related to prime numbers and factoring, using
 
 =head1 VERSION
 
-Version 0.34
+Version 0.36
 
 
 =head1 SYNOPSIS
@@ -587,12 +590,17 @@ Takes a positive number as input, and returns 1 if the input passes the
 Agrawal-Kayal-Saxena (AKS) primality test.  This is a deterministic
 unconditional primality test which runs in polynomial time for general input.
 
-In theory, AKS is extremely important.  In practice, it is essentially
-useless.  Estimated run time for a 150 digit input is about 9 years,
-making the case that while the algorithmic complexity I<growth> is
-polynomial, the constants are ludicrously high.  There are some ideas
-of Bernstein that can reduce this a little, but it would still take years
-for numbers that ECPP or APR-CL can prove in seconds.
+The particular method used is theorem 4.1 from Bernstein (2003).  This is
+substantially faster than the original AKS publication, the later version
+with improvements by Lenstra (sometimes called the V6 paper), or the later
+improvements of Voloch and Bornemann.  It is, by a large order, faster than
+any other known implementation as of early 2016.
+
+For theoretical analysis of the primality task, AKS is extremely important.
+In practice, it is essentially useless.  Estimated run time for a 150 digit
+input is over 2 days, making the case that while the algorithmic complexity
+I<growth> is polynomial, the constants are extremely high.  It will take
+years for for numbers that ECPP or APR-CL can prove in seconds.
 
 Typically you should use L</is_provable_prime> and let it decide the method.
 
@@ -724,6 +732,26 @@ that want to apply their own primality or other tests, and wish to have a
 list of values in the range with no small factors.  This is quite common
 for applications involving prime gaps.
 
+Also see L</sieve_range>.
+
+
+=head2 sieve_range
+
+  my @candidates = sieve_range(2**1000, 10000, 40000);
+
+Given a start value C<n>, and native unsigned integers C<width> and C<depth>,
+a sieve of maximum depth C<depth> is done for the C<width> consecutive
+numbers beginning with C<n>.  An array of offsets from the start is returned.
+
+The returned list contains those offsets in the range C<n> to C<n+width-1>
+where C<n + offset> has no prime factors less than C<depth>.
+
+This function is very similar to the three argument form of L</sieve_primes>.
+The differences are using C<(n,width)> instead of C<(low,high)>, and most
+importantly returning small offsets from the start value rather than the
+values themselves.  This can substantially reduce overhead for
+multi-thousand digit numbers.
+
 
 =head2 sieve_twin_primes
 
@@ -994,6 +1022,13 @@ is a list of the smallest primitive roots, which is what this function
 produces.
 
 
+=head2 is_primitive_root
+
+Given two non-negative numbers C<a> and C<n>, returns C<1> if C<a> is a
+primitive root modulo C<n>, and C<0> if not.  If C<a> is a primitive root,
+then C<euler_phi(n)> is the smallest C<e> for which C<a^e = 1 mod n>.
+
+
 =head2 sigma
 
   say "Sum of divisors of $n:", sigma( $n );
@@ -1053,6 +1088,47 @@ Given two integers C<a> and C<n>, return the inverse of C<a> modulo C<n>.
 If not defined, undef is returned.  If defined, then the return value
 multiplied by C<a> equals C<1> modulo C<n>.
 
+=head2 sqrtmod
+
+Given two integers C<a> and C<p>, return the square root of C<a> mod C<p>.
+If no square root exists, undef is returned.  If defined, the return value
+C<s> will always satisfy C<mulmod(s,s,p) = a>.
+
+If C<p> is not a prime, it is possible no result will be returned even
+though a modular root exists.
+
+Only one root is returned, even though there are at least two.  In the
+case of C<p> a prime and a return value C<s>, then both C<+s mod n> and
+C<-s mod n> are roots.  The least C<s> will be returned.  In the case of
+composites, many roots may exist, but only one will be returned.
+
+=head2 addmod
+
+Given three integers C<a>, C<b>, and C<n> where C<a> and C<n> are unsigned,
+return C<(a+b) mod n>.  This is particularly useful when dealing with
+numbers that are larger than a half-word but still native size.  No
+bigint package is needed and this can be 10-200x faster than using one.
+
+=head2 mulmod
+
+Given three integers C<a>, C<b>, and C<n> where C<a> and C<n> are unsigned,
+return C<(a*b) mod n>.  This is particularly useful when C<n> fits in a
+native integer.  No bigint package is needed and this can be 10-200x
+faster than using one.
+
+=head2 powmod
+
+Given three integers C<a>, C<b>, and C<n> where C<a> and C<n> are unsigned,
+return C<(a ** b) mod n>.  Typically binary exponentiation is used, so
+the process is very efficient.  With native size inputs, no bigint
+library is needed.
+
+=head2 divmod
+
+Given three integers C<a>, C<b>, and C<n> where C<a> and C<n> are unsigned,
+return C<(a/b) mod n>.  This is done as C<(a * (1/b mod n)) mod n>.  If
+no inverse of C<b> mod C<n> exists then undef if returned.
+
 
 =head2 consecutive_integer_lcm
 
@@ -1457,6 +1533,9 @@ There are now better alternatives.
 
 =item Robert Baillie and Samuel S. Wagstaff, Jr., "Lucas Pseudoprimes", Mathematics of Computation, v35 n152, October 1980, pp 1391-1417.  L<http://mpqs.free.fr/LucasPseudoprimes.pdf>
 
+=item Daniel J. Bernstein, "Proving Primality After Agrawal-Kayal-Saxena",
+preprint, Jan 2003.  L<http://cr.yp.to/papers/aks.pdf>
+
 =item Jon Grantham, "Frobenius Pseudoprimes", Mathematics of Computation, v70 n234, March 2000, pp 873-891.  L<http://www.ams.org/journals/mcom/2001-70-234/S0025-5718-00-01197-2/>
 
 =item John Brillhart, D. H. Lehmer, and J. L. Selfridge, "New Primality Criteria and Factorizations of 2^m +/- 1", Mathematics of Computation, v29, n130, Apr 1975, pp 620-647.  L<http://www.ams.org/journals/mcom/1975-29-130/S0025-5718-1975-0384673-1/S0025-5718-1975-0384673-1.pdf>
@@ -1516,7 +1595,7 @@ ECM implementation, as well as the papers by Brent and Montgomery.
 
 =head1 COPYRIGHT
 
-Copyright 2011-2015 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
+Copyright 2011-2016 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
